#! /usr/local/bin/ruby -Ku
# encoding: utf-8
# am_assert.rb

# =====================================================================
# Usage:
#	require_relative 'am_assert';
#	include AmAssert;
# =====================================================================

module AmAssert

	# ---------------------------------------------------------------------
	#
	def am_print_backtrace()
		caller(0).each do |trace|
			next if /am_assert\.rb/ =~ trace;
			$stderr.print "-> ", trace, "\n";
		end
	end
	private :am_print_backtrace;

	# ---------------------------------------------------------------------
	def am_assert(cond, msg = "")
		if !cond.instance_of?(TrueClass) && !cond.instance_of?(FalseClass)
			raise TypeError, "first argument should be of TrueClass or FalseClass."
		end

		return if cond == true;

		$stderr.print(msg, "\n") if msg != "";
		am_print_backtrace();
	end

	# ---------------------------------------------------------------------
	def am_assert_equals(guess, actual, msg = "")
		return if guess == actual;

		$stderr.print(msg, "\n") if msg != "";
		$stderr.print "guess  = #{guess.inspect}\nactual = #{actual.inspect}\n";
		am_print_backtrace();
	end

	# ---------------------------------------------------------------------
	def am_assert_not_equals(guess, actual, msg = "")
		return if guess != actual;

		$stderr.print(msg, "\n") if msg != "";
		$stderr.print "guess  = #{guess.inspect}\nactual = #{actual.inspect}\n";
		am_print_backtrace();
	end

	# ---------------------------------------------------------------------
	def am_assert_matches(regexp, actual, msg = "")
		return if regexp =~ actual;

		$stderr.print(msg, "\n") if msg != "";
		$stderr.print "regexp  = #{regexp.inspect}\nactual = #{actual.inspect}\n";
		am_print_backtrace();
	end

	# ---------------------------------------------------------------------
	def am_assert_array_equals_as_set(guess, actual, msg = "")
		sub_array1 = guess - actual;
		sub_array2 = actual - guess;
		return if sub_array1.empty? && sub_array2.empty?;

		$stderr.print(msg, "\n") if msg != "";
		$stderr.print "guess  = #{guess.inspect}\nactual = #{actual.inspect}\n";
		am_print_backtrace();
	end

	# ---------------------------------------------------------------------
	def am_assert_nil(actual, msg = "")
		return if actual == nil;

		$stderr.print(msg, "\n") if msg != "";
		$stderr.print "actual = #{actual.inspect}\n";
		am_print_backtrace();
	end

	# ---------------------------------------------------------------------
	# am_assert_exception(<発生するはずの例外>, msg) {
	#	例外が発生するはずのコード
	# }
	# ---------------------------------------------------------------------
	def am_assert_exception(excep, msg = "")
		exception_raised = true;
		actually_raised = "";
		begin
			yield;
			exception_raised = false;
		rescue Exception
			if $!.instance_of?(excep)
				exception_raised = true;
			else
				exception_raised = false;
				actually_raised = "#{$!.inspect}";
			end
		end
		if ! exception_raised
			$stderr.print(msg, "\n") if msg != "";
			$stderr.print "guess = #{excep.inspect}, actually_raised = #{actually_raised}\n" if ! actually_raised.empty?();
			am_print_backtrace();
		end
	end

end
