#! /usr/local/bin/ruby
# encoding: utf-8
#
# test_user_tbl.rb
#

require_relative 'am_assert';
include AmAssert;
require_relative 'pgx';

# ======================================================================
#
class TestUserTbl
	# ==============================================================
	#
	def initialize()
		@pq = Pq.new('postgresql://localhost/db_plrb');
		@pq.execute("DELETE FROM user_tbl;");
	end

	# ==============================================================
	#
	def authorized(user_id, passwd)
		sql = "SELECT login_authorized($1, $2) AS authorized;"
		return @pq.select_one(sql, user_id, passwd)['authorized'];
	end

	# ==============================================================
	#
	def test_trigers_1()
		# ======================================================
		# 新規追加。パスワードを指定していないので、「空」の
		# パスワードが登録される。
		#
		@pq.execute("INSERT INTO user_tbl (user_id) VALUES('Lucy');");

		am_assert_equals(true, authorized('Lucy', ''), 'Step 1');

		# ======================================================
		# パスワードを変更する。
		#
		@pq.execute("UPDATE user_tbl SET passwd = 'pw' WHERE user_id = 'Lucy';");
		am_assert_equals(true, authorized('Lucy', 'pw'), 'Step 2a');
		am_assert_equals(false, authorized('Lucy', ''), 'Step 2b');

		# ======================================================
		# 権限を変更する。パスワードは変わらない。
		#
		@pq.execute("UPDATE user_tbl SET privilege = 5 WHERE user_id = 'Lucy';");
		am_assert_equals(true, authorized('Lucy', 'pw'), 'Step 3a');
		am_assert_equals(false, authorized('Lucy', ''), 'Step 3b');

		rec = @pq.select_one("SELECT * FROM user_tbl WHERE user_id = 'Lucy';");
		am_assert_equals(5, rec['privilege'].to_i, 'Step 3c (privilege)');
	end

	# ==============================================================
	#
	def test_trigers_2()
		# ======================================================
		# 新規追加。パスワードも設定。
		#
		@pq.execute("INSERT INTO user_tbl (user_id, passwd, privilege) VALUES('John', 'pw_green', 15);");

		am_assert_equals(true, authorized('John', 'pw_green'), 'Step 1a');
		am_assert_equals(false, authorized('John', 'NotOW'), 'Step 1b');
		rec = @pq.select_one("SELECT * FROM user_tbl WHERE user_id = 'John';");
		am_assert_equals(15, rec['privilege'].to_i, 'Step 1c (privilege)');

		# ======================================================
		# ユーザーIDを変更する。パスワードは無効になる。
		#
		@pq.execute("UPDATE user_tbl SET user_id = 'NewJohn' WHERE user_id = 'John';");
		am_assert_equals(false, authorized('NewJohn', 'pw_green'), 'Step 2a');

		# ======================================================
		# パスワードを設定し直す。
		#
		@pq.execute("UPDATE user_tbl SET passwd = 'pw_blue' WHERE user_id = 'NewJohn';");
		am_assert_equals(true, authorized('NewJohn', 'pw_blue'), 'Step 3a');
		am_assert_equals(false, authorized('NewJohn', ''), 'Step 3b');

		rec = @pq.select_one("SELECT * FROM user_tbl WHERE user_id = 'Lucy';");
		am_assert_equals(5, rec['privilege'].to_i, 'Step 3c (privilege)');
	end

	# ==============================================================
	#
	def test_login_authorized()
		@pq.execute("DELETE FROM user_tbl;");
		@pq.execute("INSERT INTO user_tbl (user_id, passwd) VALUES ($1, $2);",
				'Sally', 'pass_brown');
		test_cases = [
			[ 'Sally', 'pass_brown', true ],
			[ 'Sally', 'BAD_PASSWD', false ],
			[ 'Sally', '',           false ],
		];
		test_cases.each do |user_id, passwd, guess_result|
			sql = "SELECT login_authorized($1, $2) AS authorized;"
			result = @pq.select_one(sql, user_id, passwd)['authorized'];
			am_assert_equals(guess_result, result, "[CASE] passwd = '#{passwd}' ");
		end
	end
end

# ======================================================================
#
if __FILE__ == $0
	t = TestUserTbl.new();
	t.test_trigers_1();
	t.test_trigers_2();
	t.test_login_authorized();
end

